#include <stdlib.h>
#include <assert.h>

#include "item.h"

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
item_t * item_alloc(item_t ** items_head)
{
    assert(items_head);

    /* always need to alloc */
    item_t * item = calloc(1, sizeof(*item));
    if(item)
    {
	/* insert before head */
	if(*items_head)
	{
	    (*items_head)->prev = item;
	}
	item->next = *items_head;
	/* this item is the new head */
	*items_head = item;
    }
    return item;
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
void item_free(item_t * item, item_t ** items_head)
{
    assert(items_head);

    /* search the item */
    item_t * curr = *items_head;
    while(curr && (curr != item))
    {
	curr = curr->next;
    }

    if(curr)
    {
	/* update previous */
	if(curr->prev)
	{
	    curr->prev->next = curr->next;
	}
	/* update next */
	if(curr->next)
	{
	    curr->next->prev = curr->prev;
	}
	/* if the item is at the head of the list move the head to the next item */
	if(*items_head == curr)
	{
	    *items_head = curr->next; 
	}
	/* can free the item now */
	free(curr);
    }
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
void item_print(item_t * item, FILE * fp, unsigned print_all)
{
    assert(item);

    do
    {
	fprintf(fp, "Item #%u (%p) : dynamic=%u, collectable=%u, removable=%u, x=%f, y=%f, a=%f, ",
		item->id,
		item,
		item->dynamic,
		item->collectable,
		item->removable,
		item->position[0],
		item->position[1],
		item->angle);
		    
	if(item->shape == Circle)
	{
	    fprintf(fp, "circle, r=%f, ", item->radius);
	}
	else if(item->shape == Box)
	{
	    fprintf(fp, "box, w=%f, h=%f, ", item->width, item->height);
	}

	fprintf(fp, "dynamic_data=%p\n", item->dynamic_data);

	item = item->next;

    } while(print_all && item);
}

