#include <stdlib.h>
#include <assert.h>
#include <math.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "render.h"

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
static const GLfloat circle_vertices[] = 
{
    0.0f   ,  0.0f,
    0.0000f,  1.0000f,
    0.2588f,  0.9659f,
    0.5000f,  0.8660f,
    0.7071f,  0.7071f,
    0.8660f,  0.5000f,
    0.9659f,  0.2588f,
    1.0000f,  0.0000f,
    0.9659f, -0.2588f,
    0.8660f, -0.5000f,
    0.7071f, -0.7071f,
    0.5000f, -0.8660f,
    0.2588f, -0.9659f,
    0.0000f, -1.0000f,
    -0.2588f, -0.9659f,
    -0.5000f, -0.8660f,
    -0.7071f, -0.7071f,
    -0.8660f, -0.5000f,
    -0.9659f, -0.2588f,
    -1.0000f, -0.0000f,
    -0.9659f,  0.2588f,
    -0.8660f,  0.5000f,
    -0.7071f,  0.7071f,
    -0.5000f,  0.8660f,
    -0.2588f,  0.9659f,
    0.0000f,  1.0000f
};

static const int circle_vertices_count = sizeof(circle_vertices)/sizeof(circle_vertices[0])/2;

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
static const GLfloat box_vertices[] = 
{
    0.0,  0.0,
    0.0,  0.5,
    0.5,  0.5,
    0.5, -0.5,
    -0.5, -0.5,
    -0.5,  0.5,
    0.0,  0.5
};

static const int box_vertices_count = sizeof(box_vertices)/sizeof(box_vertices[0])/2;

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
struct render_s
{
    world_t * world;
};

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
render_t * render_alloc(void)
{
    render_t * render;
    render = calloc(1, sizeof(*render));
    return render;
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
void render_free(render_t * render)
{
    assert(render);

    free(render);
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
void render_init(render_t * render, world_t * world)
{
    assert(render);

    render->world = world;

    glViewport( 0, 0, world->width, world->height);

    glClearColor(world->color[0], world->color[1], world->color[2], world->color[3]);

    glEnableClientState(GL_VERTEX_ARRAY);

    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    glOrtho(0, world->width, 0.0, world->height, -1.0, 1.0);
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
void render_kill(render_t * render)
{
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
void render_update(render_t * render)
{
    assert(render);

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity( );

#if !defined(NDEBUG)
    // debug squares
    glColor3f(1,0,0);
    glRectf(render->world->width - render->world->width / 40.0, 
	    0, 
	    render->world->width, 
	    render->world->height / 40.0);

    glColor3f(0,1,0);
    glRectf(0,
	    render->world->height - render->world->height / 40.0, 
	    render->world->width / 40.0, 
	    render->world->height);

    glColor3f(0,0,1);
    glRectf(0,
	    0, 
	    render->world->width / 40.0, 
	    render->world->height / 40.0);
#endif

    /* for each items */
    item_t * item = render->world->items_head;
    while(item)
    {
	/* prepare vertex list */
	unsigned vertices_count;
	if(item->shape == Circle)
	{
	    glVertexPointer(2, GL_FLOAT, 0, circle_vertices);
	    vertices_count = circle_vertices_count;
	}
	else if(item->shape == Box)
	{
	    glVertexPointer(2, GL_FLOAT, 0, box_vertices);
	    vertices_count = box_vertices_count;
	}
	else
	{
	    continue;
	}

	/* draw */
	glPushMatrix();
	glPushAttrib(GL_LINE_BIT | GL_CURRENT_BIT);
	glTranslated(item->position[0], item->position[1], 0);
	glRotated(item->angle * 180.0 / M_PI, 0, 0, 1);
	glScaled(item->width, item->height, 1.0);
	glColor3dv(item->color);
	glDrawArrays(GL_TRIANGLE_FAN, 0, vertices_count);
	glColor3f(0,0,0);
	if(item->collectable)
	{
	    glLineWidth(2);
	}
	if(item->removable)
	{
	    glEnable(GL_LINE_STIPPLE);
	    glLineStipple(2, 0xAAAA);
	}
	glDrawArrays(GL_LINE_STRIP, 0, vertices_count);
	if(item->removable)
	{
	    glDisable(GL_LINE_STIPPLE);
	}
	glPopAttrib();
	glPopMatrix();

	/* next one ! */
	item = item->next;
    }	
} 


