#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include <chipmunk.h>

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
static void shape_free(cpBody *body, cpShape *shape, cpSpace * space)
{
    assert(body && shape && space);

    if(cpSpaceContainsShape(space, shape))
    {
	cpSpaceRemoveShape(space, shape);
    }

    cpShapeFree(shape);
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
static void body_free(cpBody *body, cpSpace * space)
{
    assert(body && space);

    cpBodyEachShape(body, (cpBodyShapeIteratorFunc)shape_free, space);

    if(cpSpaceContainsBody(space, body))
    {
	cpSpaceRemoveBody(space, body);
    }

    cpBodyFree(body);
}

/*****************************************************************************
 *                                                                           *
 *****************************************************************************/
#define NB_BODIES 10
#define C 10.0
#define DT (1.0 / 60.0)

int main(int argc, char **argv)
{
    cpSpace * space;
    cpBody * body[NB_BODIES];
    cpFloat area = cpAreaForCircle(0, C);
    cpFloat moment = cpMomentForCircle(area, 0, C, cpvzero);
    unsigned count = 0;
    int i;

    srand(0);

    cpInitChipmunk();
    space = cpSpaceNew();
    cpSpaceSetGravity(space, cpv(0, -80));

    for(i = 0; i < NB_BODIES; i++)
    {
	body[i] = cpBodyNew(area, moment);
	cpSpaceAddBody(space, body[i]);
	cpBodySetPos(body[i], cpv(rand() % 100, rand() % 100));

	cpShape * shape;
	shape = cpCircleShapeNew(body[i], C, cpvzero);
	cpSpaceAddShape(space, shape);

	count++;
    }

    while(count)
    {
	cpSpaceStep(space, DT);

	for(i = 0; i < NB_BODIES; i++)
	{
	    if(body[i])
	    {
		cpVect pos = cpBodyGetPos(body[i]);
		if(pos.y < -1.0)
		{
		    body_free(body[i], space);
		    body[i] = NULL;
		    count--;
		}
	    }
	}
    }

    cpSpaceFree(space);

    return 0;
}
